package com.dragonsoft.dcuc.approve.facade;

import com.dragonsoft.dcuc.approve.model.req.*;
import com.dragonsoft.dcuc.approve.model.resp.*;
import com.dragonsoft.duceap.base.annotations.parameter.QueryObject;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.duwf.api.model.ActivityNodeInfoVo;
import com.dragonsoft.duceap.duwf.api.model.TraceProcessAndOverVo;
import com.dragonsoft.duceap.duwf.api.model.WorkflowProcesstypeVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.data.domain.Page;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotBlank;
import java.util.List;

/**
 * @Author: wang.rs
 * @Date: 2020-12-15
 * @Description:
 */
@Api(tags = "审批服务")
@FeignClient(name = "approve")
public interface IApproveFacade {

    /**
     * 申请统计
     *
     * @param searchable 查询条件
     * @return 状态
     */
    @ApiOperation(value = "申请统计", notes = "申请统计")
    @RequestMapping(value = "v1/apply-statistics/search", method = RequestMethod.POST)
    ResponseDTO<StatisticsDataVo> applyStatistics(@QueryObject Searchable searchable);

    /**
     * 我的请求
     *
     * @param searchable 查询条件
     * @return 分页
     */
    @ApiOperation(value = "我的请求列表", notes = "我的请求列表")
    @RequestMapping(value = "v1/user-apply-list/search", method = RequestMethod.POST)
    Page<ApprovalInfoRespVO> queryUserApplyList(@QueryObject Searchable searchable);

    /**
     * 待办已办
     *
     * @param searchable 查询条件
     * @return 分页
     */
    @ApiOperation(value = "查询已办或待办列表", notes = "查询已办或代办列表")
    @RequestMapping(value = "v1/approve-list/search", method = RequestMethod.POST)
    Page<TaskVo> queryApproveList(@QueryObject Searchable searchable);

    /**
     * 删除我的请求
     *
     * @param id id
     * @return 状态
     */
    @ApiOperation(value = "我的请求列表-删除", notes = "我的请求列表-删除")
    @RequestMapping(value = "v1/delApply/{id}", method = RequestMethod.DELETE)
    ResponseStatus delApply(@PathVariable(value = "id") String id);

    /**
     * 审批操作
     *
     * @param approveOperateReqVO 操作信息
     * @return 状态
     */
    @ApiOperation(value = "审批操作", notes = "审批操作")
    @RequestMapping(value = "v1/approve-operate", method = RequestMethod.PUT)
    ResponseDTO<OperateRespVO> approveOperate(@Validated @RequestBody ApproveOperateReqVO approveOperateReqVO);

    /**
     * 审批操作提示
     *
     * @param processInstanceId 流程实例ID
     * @return 状态
     */
    @ApiOperation(value = "审批操作提示", notes = "审批操作提示")
    @RequestMapping(value = "v1/approve-operate-tips/{processInstanceId}", method = RequestMethod.GET)
    ResponseDTO<List<OperateTipResp>> approveOperatePreTip(@Validated @NotBlank @PathVariable String processInstanceId);

    /**
     * 查看红名单状态
     *
     * @param approveNo 审批单号
     * @return 查看红名单状态
     */
    @ApiOperation(value = "查看红名单状态", notes = "查看红名单状态")
    @GetMapping("v1/red-list/{approveNo}/status")
    ResponseDTO<OperateRespVO> redListStatus(@PathVariable String approveNo);

    /**
     * 红名单审批操作
     *
     * @param approveOperateReqVO 操作信息
     * @return 状态
     */
    @ApiOperation(value = "红名单审批操作", notes = "红名单审批操作")
    @RequestMapping(value = "v1/red-list/approve-operate", method = RequestMethod.PUT)
    ResponseDTO<OperateRespVO> redListApproveOperate(@Validated @RequestBody ApproveOperateReqVO approveOperateReqVO);

    /**
     * 申请详情
     *
     * @param id id
     * @return 状态
     */
    @ApiOperation(value = "申请详情", notes = "申请详情")
    @RequestMapping(value = "v1/apply/{id}", method = RequestMethod.GET)
    ResponseDTO<ApprovalInfoRespVO> applyDetail(@PathVariable(value = "id") String id);

    /**
     * 保存、保存并提交
     *
     * @param approvalInfoReqVO 申请单信息
     * @return 状态
     */
    @ApiOperation(value = "保存或保存并提交", notes = "保存或保存并提交")
    @RequestMapping(value = "v1/apply", method = RequestMethod.POST)
    ResponseDTO<String> save(@Validated @RequestBody ApprovalInfoReqVO approvalInfoReqVO);

    /**
     * 修改
     *
     * @param approvalInfoDTO 审批信息
     * @return 状态
     */
    @ApiOperation(value = "修改", notes = "修改")
    @RequestMapping(value = "v1/apply", method = RequestMethod.PUT)
    ResponseDTO<String> update(@Validated @RequestBody ApprovalInfoUpdateReqVO approvalInfoDTO);

    /**
     * 转办
     *
     * @param agentReqVO 转办信息
     * @return 状态
     */
    @ApiOperation(value = "转办", notes = "转办")
    @RequestMapping(value = "v1/apply/agent", method = RequestMethod.PUT)
    ResponseStatus agent(@Validated @RequestBody AgentReqVO agentReqVO);

    /**
     * 审批意见
     *
     * @param approveRecordId 业务ID
     * @return 状态
     */
    @ApiOperation(value = "审批意见", notes = "审批意见")
    @RequestMapping(value = "v1/apply-info/{approveRecordId}/approve-comments", method = RequestMethod.GET)
    ResponseDTO<List<ApproveCommentVo>> approveComments(@PathVariable(value = "approveRecordId") String approveRecordId);

    /**
     * 获取流程图
     *
     * @param proInsId 实例ID
     * @return 状态
     */
    @ApiOperation(value = "获取流程图", notes = "获取流程图")
    @RequestMapping(value = "v1/apply-info/{proInsId}/process-pic", method = RequestMethod.GET)
    ResponseDTO<TraceProcessAndOverVo> processPic(@PathVariable(value = "proInsId") String proInsId);

    /**
     * 获取流程图节点详情
     *
     * @param proInsId         实例ID
     * @param statusType       状态类型
     * @param taskDefinitionId 任务节点定义ID
     * @return 状态
     */
    @ApiOperation(value = "获取流程图节点详情", notes = "获取流程图节点详情")
    @RequestMapping(value = "v1/process-pic-detail", method = RequestMethod.GET)
    ResponseDTO<ActivityNodeInfoVo> processPicDetail(@RequestParam(value = "processInstanceId") String proInsId,
                                                     @RequestParam(value = "statusType") String statusType,
                                                     @RequestParam(value = "taskDefinitionId") String taskDefinitionId);

    /**
     * 获取流程类型列表
     *
     * @return 状态
     */
    @ApiOperation(value = "获取流程类型列表", notes = "获取流程类型列表")
    @RequestMapping(value = "v1/process-types", method = RequestMethod.GET)
    ResponseDTO<List<ApproveWorkflowProcessTypeVo>> queryProcessTypeList();

    /**
     * 获取流程类型详情
     *
     * @param processTypeCode 流程类型代码
     * @return 状态
     */
    @ApiOperation(value = "获取流程类型详情", notes = "获取流程类型详情")
    @RequestMapping(value = "v1/process-type/{processTypeCode}", method = RequestMethod.GET)
    ResponseDTO<WorkflowProcesstypeVo> queryProcessType(@PathVariable(value = "processTypeCode") String processTypeCode);

    /**
     * 流程图状态
     *
     * @param approveRecordId 审批单号ID
     * @return 流程图状态
     */
    @ApiOperation(value = "流程图状态", notes = "流程图状态")
    @RequestMapping(value = "v1/apply-info/{approveRecordId}/processStatus", method = RequestMethod.GET)
    ResponseDTO<List<ProcessStatusVo>> processStatus(@PathVariable(value = "approveRecordId") String approveRecordId);

    /**
     * 获取任务支持的操作对象信息
     *
     * @param taskId   任务ID
     * @param pageType 页面类型
     * @return 状态
     */
    @GetMapping(value = "v1/task-operate/{taskId}/{pageType}")
    ResponseDTO<TaskOperateRespVO> taskOperateForApprove(@PathVariable("taskId") String taskId, @PathVariable("pageType") String pageType);

    /**
     * taskNo查找审批意见
     *
     * @param taskNo 任务编号
     * @return 状态
     */
    @ApiOperation(value = "taskNo查找审批意见", notes = "taskNo查找审批意见")
    @RequestMapping(value = "v1/apply-info-task/{taskNo}/approve-comments", method = RequestMethod.GET)
    ResponseDTO<List<ApproveCommentVo>> taskApproveComments(@PathVariable(value = "taskNo") String taskNo);

    /**
     * 申请详情
     *
     * @param taskNo 任务编号
     * @return 状态
     */
    @ApiOperation(value = "taskNo申请详情", notes = "taskNo申请详情")
    @RequestMapping(value = "v1/apply-info-task/{taskNo}", method = RequestMethod.GET)
    ResponseDTO<ApprovalInfoRespVO> taskApplyDetail(@PathVariable(value = "taskNo") String taskNo);

    /**
     * 审批操作
     *
     * @param taskApproveOperateReqVO 操作信息
     * @return 状态
     */
    @ApiOperation(value = "taskNo审批操作", notes = "taskNo审批操作")
    @RequestMapping(value = "v1/task-approve-operate", method = RequestMethod.PUT)
    ResponseDTO<OperateRespVO> taskApproveOperate(@Validated @RequestBody TaskApproveOperateReqVO taskApproveOperateReqVO);

    /**
     * 删除我的请求
     *
     * @param taskNo taskNo
     * @return 状态
     */
    @ApiOperation(value = "我的请求列表-删除", notes = "我的请求列表-删除")
    @RequestMapping(value = "v1/task-delApply/{taskNo}", method = RequestMethod.DELETE)
    ResponseStatus taskDelApply(@PathVariable(value = "taskNo") String taskNo);

}