package com.dragonsoft.dcuc.approve.facade.api;

import com.dragoninfo.dcuc.common.entity.ApiResult;
import com.dragoninfo.dcuc.common.entity.ApiSearchReq;
import com.dragonsoft.dcuc.approve.enumresources.PageTypeEnum;
import com.dragonsoft.dcuc.approve.model.req.ApprovalInfoAuthReqVO;
import com.dragonsoft.dcuc.approve.model.req.ApprovalInfoAuthUpdateReqVO;
import com.dragonsoft.dcuc.approve.model.req.v2.ApproveOperateV2ReqVO;
import com.dragonsoft.dcuc.approve.model.resp.ApprovalInfoV2RespVO;
import com.dragonsoft.dcuc.approve.model.resp.OperateRespVO;
import com.dragonsoft.dcuc.approve.model.resp.ProcessStatusVo;
import com.dragonsoft.dcuc.approve.model.resp.TaskOperateRespVO;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.duwf.api.model.ActivityNodeInfoVo;
import com.dragonsoft.duceap.duwf.api.model.TraceProcessAndOverVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 审批接口
 *
 * @author huangzqa
 * @date : 2021-07-16
 */
@Api(tags = "审批服务接口V2")
@FeignClient(name = "approve")
public interface IApiV2ApproveFacade {
    /**
     * 权限审批请求推送接口
     *
     * @param approvalInfoReqVO 审批请求信息
     * @return 状态
     */
    @ApiOperation(value = "权限审批请求推送接口", notes = "审批请求推送接口")
    @PostMapping(value = "api/v2/approves/authorities")
    ResponseDTO<String> approveAuthorities(@Validated @RequestBody ApprovalInfoAuthReqVO approvalInfoReqVO);

    /**
     * 修改权限审批单
     *
     * @param approvalInfoAuthUpdateReqVO 审批信息
     * @return 状态
     */
    @ApiOperation(value = "修改权限审批单", notes = "修改权限审批单")
    @PutMapping("api/v2/approves/authorities")
    ResponseStatus updateApprovesAuthorities(@Validated @RequestBody ApprovalInfoAuthUpdateReqVO approvalInfoAuthUpdateReqVO);

    /**
     * 查询审批流程列表
     *
     * @param apiSearchReq 查询条件
     * @return 列表
     */
    @ApiOperation(value = "查询审批流程列表", notes = "查询审批流程列表")
    @PostMapping(value = "api/v2/approves/search")
    ApiResult approvesSearch(@RequestBody ApiSearchReq apiSearchReq);

    /**
     * 查询审批流程详情
     *
     * @param approveNo 审批单号
     * @return 审批流程信息
     */
    @ApiOperation(value = "查询审批流程详情", notes = "查询审批流程详情")
    @GetMapping(value = "api/v2/approves/{approveNo}")
    ResponseDTO<ApprovalInfoV2RespVO> approveDetail(@PathVariable(value = "approveNo") String approveNo);

    /**
     * 撤销审批流程
     *
     * @param approveNo 审批单号
     * @return 结果
     */
    @ApiOperation(value = "撤销审批流程", notes = "撤销审批流程")
    @PutMapping(value = "api/v2/approves/{approveNo}/recall")
    ResponseStatus recallApprove(@PathVariable String approveNo);

    /**
     * 删除审批流程
     *
     * @param approveNo 审批单号
     * @return 状态
     */
    @ApiOperation(value = "删除审批流程", notes = "删除审批流程")
    @DeleteMapping(value = "api/v2/approves/{approveNo}")
    ResponseStatus deleteApprove(@PathVariable String approveNo);

    /**
     * 获取审批意见列表
     *
     * @param approveNo 审批单号
     * @return 审批意见列表
     */
    @ApiOperation(value = "审批意见", notes = "审批意见")
    @GetMapping(value = "api/v2/approves/{approveNo}/approve-comments")
    ResponseStatus approveComments(@PathVariable(value = "approveNo") String approveNo);

    /**
     * 获取流程图
     *
     * @param approveNo 审批单号
     * @return 流程图
     */
    @ApiOperation(value = "获取流程图", notes = "获取流程图")
    @GetMapping(value = "api/v2/approves/{approveNo}/process-pic")
    ResponseDTO<TraceProcessAndOverVo> processPic(@PathVariable(value = "approveNo") String approveNo);

    /**
     * 获取流程节点详情
     *
     * @param approveNo          审批单号
     * @param taskDefinitionCode 任务节点定义id
     * @param statusType         流程状态
     * @return 流程图
     */
    @ApiOperation(value = "获取流程节点详情", notes = "获取流程节点详情")
    @GetMapping(value = "api/v2/approves/{approveNo}/task-definitions/{taskDefinitionCode}/status/{statusType}")
    ResponseDTO<ActivityNodeInfoVo> processPicDetail(@PathVariable String approveNo, @PathVariable String taskDefinitionCode,
                                                     @PathVariable String statusType);

    /**
     * 获取流程图状态
     *
     * @param approveNo 审批单号
     * @return 流程图
     */
    @ApiOperation(value = "获取流程图状态", notes = "流程图状态")
    @GetMapping(value = "api/v2/approves/{approveNo}/process-status")
    ResponseDTO<List<ProcessStatusVo>> processStatus(@PathVariable(value = "approveNo") String approveNo);

    /**
     * 获取任务支持的操作对象信息
     *
     * @param approveNo 审批单号
     * @param pageType  页面类型
     * @return 支持的操作对象
     * @see PageTypeEnum pageType
     */
    @ApiOperation(value = "获取任务支持的操作对象信息", notes = "获取任务支持的操作对象信息")
    @RequestMapping(value = "api/v2/approves/{approveNo}/operate-type/{pageType}", method = RequestMethod.GET)
    ResponseDTO<TaskOperateRespVO> approveOperateType(@PathVariable(value = "approveNo") String approveNo,
                                                      @PathVariable(value = "pageType") String pageType);

    /**
     * 审批操作
     *
     * @param approveOperateV2ReqVO 审批操作
     * @return 结果
     */
    @ApiOperation(value = "审批操作", notes = "审批操作")
    @RequestMapping(value = "api/v2/approve-operate", method = RequestMethod.PUT)
    ResponseDTO<OperateRespVO> operate(@Validated @RequestBody ApproveOperateV2ReqVO approveOperateV2ReqVO);



    //TODO add wanghao   挂起 suspend
    /**
     * 挂起审批流程
     *
     * @param approveNo 审批单号
     * @return 结果
     */
    @ApiOperation(value = "挂起审批流程", notes = "挂起审批流程")
    @PutMapping(value = "api/v2/approves/{approveNo}/suspend")
    ResponseStatus suspendApprove(@PathVariable String approveNo);


    /**
     * 终止审批流程
     *
     * @param approveNo 审批单号
     * @return 结果
     */
    @ApiOperation(value = "终止审批流程", notes = "终止审批流程")
    @PutMapping(value = "api/v2/approves/{approveNo}/redo")
    ResponseStatus redoApprove(@PathVariable String approveNo);

    /**
     * 激活审批流程
     *
     * @param approveNo 审批单号
     * @return 结果
     */
    @ApiOperation(value = "激活审批流程", notes = "激活审批流程")
    @PutMapping(value = "api/v2/approves/{approveNo}/activate")
    ResponseStatus activateApprove(@PathVariable String approveNo);
    //TODO add wanghao   end
}