package com.dragonsoft.dcuc.approve.business.external.impl;

import cn.hutool.core.util.StrUtil;
import com.dragoninfo.dcuc.common.entity.*;
import com.dragonsoft.auditlog.collection.qmtj.provider.IRestTemplateProvider;
import com.dragonsoft.dcuc.approve.business.external.IDcucAuthBusiness;
import com.dragonsoft.dcuc.approve.constants.ApproveConstants;
import com.dragonsoft.dcuc.approve.enumresources.FlowTypeEnum;
import com.dragonsoft.dcuc.approve.enumresources.ListLevelEnum;
import com.dragonsoft.dcuc.approve.enumresources.ListResourceTypeEnum;
import com.dragonsoft.dcuc.approve.model.dto.ResourceInfoDTO;
import com.dragonsoft.dcuc.approve.model.req.AppFunctionResourceReqVO;
import com.dragonsoft.dcuc.approve.model.resp.OperateRespVO;
import com.dragonsoft.dcuc.approve.model.vo.AppFunRedListVo;
import com.dragonsoft.dcuc.approve.model.vo.AuthAppFunApiVO;
import com.dragonsoft.dcuc.approve.model.vo.WhiteListUserApiVo;
import com.dragonsoft.dcuc.approve.properties.ApproveProperties;

import com.dragonsoft.dcuc.approve.utils.ProofsUtil;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 权限中心业务管理
 * </p>
 *
 * @author huangzqa
 * @date 2021/7/8
 */
@Slf4j
@Service
public class DcucAuthBusinessImpl implements IDcucAuthBusiness {

    @Autowired
    private ApproveProperties approveProperties;

    /**
     * 红名单接口
     */
    public static final String RED_LIST_URL = "/api/list-service/v1/list/func-red-list/";
    /**
     * 白名单接口
     */
    public static final String WHITE_LIST_URL = "/api/list-service/v1/list/user-white-list/";

    /**
     * 获取应用功能列表接口
     */
    public static final String APP_FUNCTION_URL = "/api/auth-service/v1/app-functions/search";

    @Autowired
    private IRestTemplateProvider restTemplateProvider;

    /**
     * 获取红名单等级
     *
     * @param flowTypeEnum 流程类型
     * @param resourceCode 资源代码
     * @return 等级
     */
    @Override
    public Set<ListLevelEnum> getRedListLevel(FlowTypeEnum flowTypeEnum, String resourceCode) {
        ListResourceTypeEnum listResourceTypeEnum = flowTypeEnum.toListResourceTypeEnum();

        if (listResourceTypeEnum == ListResourceTypeEnum.FUNCTION) {
            return getRedFunctionListLevel(resourceCode);
        }

        return Collections.emptySet();
    }

    /**
     * 请求LEVEL
     *
     * @param visitResourceCode 资源代码
     * @return 资源等级
     */
    public Set<ListLevelEnum> getRedFunctionListLevel(String visitResourceCode) {

        ObjectMapper objectMapper = new ObjectMapper();
        List<AppFunctionResourceReqVO> appFunctionResourceReqList = Collections.emptyList();
        try {
            appFunctionResourceReqList = objectMapper.readValue(visitResourceCode, new TypeReference<List<AppFunctionResourceReqVO>>() {
            });
        } catch (JsonProcessingException e) {
            log.error("parse error", e);
        }
        Set<ListLevelEnum> totalListLevelEnum = new HashSet<>();
        for (AppFunctionResourceReqVO appFunctionResourceReqVO : appFunctionResourceReqList) {
            String appCode = appFunctionResourceReqVO.getAppCode();
            List<String> functionCode = appFunctionResourceReqVO.getFunctionCode();
            Set<ListLevelEnum> listLevelEnumSet = getFunctionLevel(appCode, functionCode);
            totalListLevelEnum.addAll(listLevelEnumSet);
        }

        return totalListLevelEnum;
    }

    @SneakyThrows(JsonProcessingException.class)
    private Set<ListLevelEnum> getFunctionLevel(String appCode, List<String> functionCodeArray) {
        HttpHeaders httpHeaders = ProofsUtil.buildHttpHeaders();
        log.debug("header:{}", httpHeaders);

        Map<String, SearchParam> filters = new HashMap<>(1);
        SearchParam searchParamCode = new SearchParam();
        searchParamCode.setOperator(SearchOperator.eq.name());
        searchParamCode.setValue(appCode);
        SearchParam functionSearchParamCode = new SearchParam();
        functionSearchParamCode.setOperator(SearchOperator.in.name());
        functionSearchParamCode.setValue(functionCodeArray.toArray());

        filters.put("appCode", searchParamCode);
        filters.put("funCode", functionSearchParamCode);
        ApiSearchReq apiSearchReq = new ApiSearchReq();

        apiSearchReq.setFilters(filters);

        HttpEntity<ApiSearchReq> requestEntity = new HttpEntity<>(apiSearchReq, httpHeaders);
        String reqUrl = approveProperties.getAuthServiceUrl() + RED_LIST_URL;
        RestTemplate restTemplate = restTemplateProvider.getRestTemplate();
        ResponseEntity<ApiResult> resultResponseEntity = restTemplate.exchange(reqUrl, HttpMethod.POST, requestEntity, ApiResult.class);
        HttpStatus statusCode = resultResponseEntity.getStatusCode();
        log.debug("getFunctionLevel resp:{}", JsonUtils.toJSONString(resultResponseEntity));
        if (statusCode.is2xxSuccessful()) {
            ApiResult apiResult = resultResponseEntity.getBody();
            if (apiResult == null) {
                return Collections.emptySet();
            }

            String respStatusCode = apiResult.getStatusCode();
            if (!respStatusCode.equals(ResponseStatus.SUCCESS_CODE)) {
                return Collections.emptySet();
            }

            Object result = apiResult.getResult();
            String jsonString = JsonUtils.toJSONString(result);

            ObjectMapper objectMapper = new ObjectMapper();
            List<AppFunRedListVo> content = objectMapper.readValue(jsonString, new TypeReference<List<AppFunRedListVo>>() {
            });

            return content.stream().map(item -> ListLevelEnum.valueOfSort(Integer.parseInt(item.getLevel())))
                    .collect(Collectors.toSet());
        } else {
            log.error("request error:{}", JsonUtils.toJSONString(resultResponseEntity));
        }
        return Collections.emptySet();
    }

    @Override
    @SneakyThrows(JsonProcessingException.class)
    public List<AuthAppFunApiVO> getFunctionList(String appCode, List<String> functionCodList) {
        
        HttpHeaders httpHeaders = ProofsUtil.buildHttpHeaders();
        log.debug("header:{}", httpHeaders);

        Map<String, SearchParam> filters = new HashMap<>(1);
        SearchParam searchParamCode = new SearchParam();
        searchParamCode.setOperator(SearchOperator.eq.name());
        searchParamCode.setValue(appCode);

        SearchParam searchParamFuncCode = new SearchParam();
        searchParamFuncCode.setOperator(SearchOperator.in.name());
        searchParamFuncCode.setValue(functionCodList);

        filters.put("appCode", searchParamCode);
        filters.put("funCode", searchParamFuncCode);

        ApiSearchReq apiSearchReq = new ApiSearchReq();

        apiSearchReq.setFilters(filters);

        HttpEntity<ApiSearchReq> requestEntity = new HttpEntity<>(apiSearchReq, httpHeaders);
        String reqUrl = approveProperties.getAuthServiceUrl() + APP_FUNCTION_URL;
        RestTemplate restTemplate = restTemplateProvider.getRestTemplate();
        ResponseEntity<ApiResult> resultResponseEntity = restTemplate.exchange(reqUrl, HttpMethod.POST, requestEntity, ApiResult.class);
        HttpStatus statusCode = resultResponseEntity.getStatusCode();
        log.debug("getFunctionResource resp:{}", JsonUtils.toJSONString(resultResponseEntity));
        if (statusCode.is2xxSuccessful()) {
            ApiResult apiResult = resultResponseEntity.getBody();
            if (apiResult == null) {
                return Collections.emptyList();
            }

            String respStatusCode = apiResult.getStatusCode();
            if (!respStatusCode.equals(ResponseStatus.SUCCESS_CODE)) {
                return Collections.emptyList();
            }

            Object result = apiResult.getResult();
            String jsonString = JsonUtils.toJSONString(result);

            ObjectMapper objectMapper = new ObjectMapper();

            return objectMapper.readValue(jsonString, new TypeReference<List<AuthAppFunApiVO>>() {
            });
        } else {
            log.error("request error:{}", JsonUtils.toJSONString(resultResponseEntity));
        }
        return Collections.emptyList();
    }

    /**
     * 请求LEVEL
     *
     * @param resourceCode 资源代码
     * @return 资源等级
     */
    @Override
    @SneakyThrows(JsonProcessingException.class)
    public Optional<ListLevelEnum> getWhiteListLevel(String resourceCode) {
        
        HttpHeaders httpHeaders = ProofsUtil.buildHttpHeaders();
        log.debug("header:{}", httpHeaders);

        Map<String, SearchParam> filters = new HashMap<>(1);
        SearchParam searchParamCode = new SearchParam();
        searchParamCode.setOperator(SearchOperator.eq.name());
        searchParamCode.setValue(resourceCode);
        filters.put("idcard", searchParamCode);

        ApiSearchReq apiSearchReq = new ApiSearchReq();

        ApiPageReq apiPageReq = new ApiPageReq();
        apiPageReq.setFrom(1);
        apiPageReq.setSize(1);
        apiSearchReq.setPage(apiPageReq);
        apiSearchReq.setFilters(filters);

        HttpEntity<ApiSearchReq> requestEntity = new HttpEntity<>(apiSearchReq, httpHeaders);
        String reqUrl = approveProperties.getAuthServiceUrl() + WHITE_LIST_URL;
        RestTemplate restTemplate = restTemplateProvider.getRestTemplate();
        ResponseEntity<ApiResult> resultResponseEntity = restTemplate.exchange(reqUrl, HttpMethod.POST, requestEntity, ApiResult.class);
        log.debug("getWhiteListLevel resp:{}", JsonUtils.toJSONString(resultResponseEntity));
        HttpStatus statusCode = resultResponseEntity.getStatusCode();
        if (statusCode.is2xxSuccessful()) {
            ApiResult apiResult = resultResponseEntity.getBody();
            if (apiResult == null) {
                return Optional.empty();
            }
            String respStatusCode = apiResult.getStatusCode();
            if (!respStatusCode.equals(ResponseStatus.SUCCESS_CODE)) {
                return Optional.empty();
            }
            Object result = apiResult.getResult();
            String jsonString = JsonUtils.toJSONString(result);
            ObjectMapper objectMapper = new ObjectMapper();
            List<WhiteListUserApiVo> content = objectMapper.readValue(jsonString, new TypeReference<List<WhiteListUserApiVo>>() {
            });
            if (content.isEmpty()) {
                return Optional.empty();
            }
            WhiteListUserApiVo whiteListUserApiVo = content.get(0);
            String level = whiteListUserApiVo.getListLevel();
            ListLevelEnum listLevelEnum = ListLevelEnum.valueOfSort(Integer.parseInt(level));
            return Optional.of(listLevelEnum);
        } else {
            log.error("request error:{}", JsonUtils.toJSONString(resultResponseEntity));
        }
        return Optional.empty();
    }

    @Override
    public OperateRespVO checkHitRedList(String checkIdcard, FlowTypeEnum flowTypeEnum, ResourceInfoDTO resourceInfoDTO) {
        // 获取主体白名单
        ListLevelEnum subjectListLevelEnum = getWhiteListLevel(checkIdcard).orElse(ListLevelEnum.NORMAL);
        // 高级别的白名单可以访问所有不需要判断
        if (subjectListLevelEnum.equals(ListLevelEnum.HIGH)) {
            return OperateRespVO.notRequired();
        }
        // 获取客体红名单
        String visitResourceCode = resourceInfoDTO.getVisitResourceCode();
        Set<ListLevelEnum> objectListLevelEnumSet = new HashSet<>();
        if (StrUtil.isNotBlank(visitResourceCode)) {
            objectListLevelEnumSet.addAll(getRedListLevel(flowTypeEnum, visitResourceCode));
        }
        // 撤销资源不进行红名单判断

        ListLevelEnum maxLevel = getMaxLevel(objectListLevelEnumSet);
        // 普通资源可以被所有访问无需判断
        if (maxLevel.equals(ListLevelEnum.NORMAL)) {
            return OperateRespVO.notRequired();
        }

        Integer subjectListLevelEnumSort = subjectListLevelEnum.getSort();
        Integer maxLevelSort = maxLevel.getSort();

        if (subjectListLevelEnumSort < maxLevelSort) {
            String subTipString = ListResourceTypeEnum.USER.getLabel();
            if (!subjectListLevelEnum.equals(ListLevelEnum.NORMAL)) {
                subTipString = subjectListLevelEnum.getLabel() + "白名单" + subTipString;
            }
            String tip = String.format(ApproveConstants.LIST_TIP, subTipString, maxLevel.getLabel(), flowTypeEnum.getResourceName());
            return OperateRespVO.need(tip);
        }
        return OperateRespVO.notRequired();
    }

    /**
     * 获取最大的风险基本
     *
     * @param listLevelEnums 风险级别列表
     * @return 最大风险级别
     */
    public ListLevelEnum getMaxLevel(Set<ListLevelEnum> listLevelEnums) {
        // 判断规则
        Optional<Integer> maxSort = listLevelEnums.stream()
                .map(ListLevelEnum::getSort)
                .max(Integer::compareTo);

        return ListLevelEnum.valueOfSort(maxSort.orElse(0));
    }

}
