package com.dragonsoft.dcuc.approve.business.external.impl;

import cn.hutool.core.util.StrUtil;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragoninfo.dcuc.common.utils.ValidUtil;
import com.dragonsoft.dcuc.approve.business.external.INoticeBusiness;
import com.dragonsoft.dcuc.approve.constants.ApproveBeanConstants;
import com.dragonsoft.dcuc.approve.enumresources.bu.BuErrorCodeEnum;
import com.dragonsoft.dcuc.approve.model.req.v3.ZeroTrustApproveCallbackReqV3VO;
import com.dragonsoft.dcuc.approve.model.req.bu.BuApproveSyncReqVO;
import com.dragonsoft.dcuc.approve.model.resp.v3.ZeroTrustMessageRespVO;
import com.dragonsoft.dcuc.approve.model.resp.v3.ZeroTrustResultDataVO;
import com.dragonsoft.dcuc.approve.properties.ApproveBuProperties;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.rholder.retry.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.RequestEntity;
import org.springframework.http.ResponseEntity;
import org.springframework.remoting.RemoteAccessException;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.net.URI;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;

/**
 * @author hunagzqa
 * @date 2022/11/16
 */
@Component
public class NoticeBusinessImpl implements INoticeBusiness {

    private RestTemplate restTemplate;

    private ObjectMapper dragonObjectMapper;

    private ApproveBuProperties approveBuProperties;


    @Qualifier(ApproveBeanConstants.SKIP_SSL_REST_TEMPLATE)
    @Autowired
    public void setRestTemplate(RestTemplate restTemplate) {
        this.restTemplate = restTemplate;
    }

    @Qualifier(ApproveBeanConstants.DRAGON_OBJECT_MAPPER)
    @Autowired
    public void setDragonObjectMapper(ObjectMapper dragonObjectMapper) {
        this.dragonObjectMapper = dragonObjectMapper;
    }

    @Autowired
    public void setApproveBuProperties(ApproveBuProperties approveBuProperties) {
        this.approveBuProperties = approveBuProperties;
    }

    private static final Logger logger = LoggerFactory.getLogger(NoticeBusinessImpl.class);

    private static final Retryer<ResponseStatus> RESPONSE_STATUS_RETRYER =
            RetryerBuilder.<ResponseStatus>newBuilder()
                    .retryIfException()
                    .retryIfResult(item -> !item.getStatusCode().equals(ResponseStatus.SUCCESS_CODE))
                    .withAttemptTimeLimiter(AttemptTimeLimiters.fixedTimeLimit(5, TimeUnit.SECONDS))
                    .withWaitStrategy(WaitStrategies.fixedWait(1, TimeUnit.SECONDS))
                    .withStopStrategy(StopStrategies.stopAfterAttempt(5))
                    .build();


    @Override
    public ResponseStatus retryApproveNotification(String callbackUrl, ZeroTrustApproveCallbackReqV3VO zeroTrustApproveCallbackReqV3VO) {
        StringBuilder errorMessageBuilder = new StringBuilder();

        ResponseStatus responseStatus;
        try {
            responseStatus = RESPONSE_STATUS_RETRYER.call(() -> baseApproveNotification(callbackUrl, zeroTrustApproveCallbackReqV3VO));
        } catch (ExecutionException | RetryException e) {
            logger.error("retry error", e);
            responseStatus = ResponseStatus.fail(e.getMessage());
        }
        if (!responseStatus.getStatusCode().equals(ResponseStatus.SUCCESS_CODE)) {
            errorMessageBuilder.append("URL: ").append(callbackUrl)
                    .append(responseStatus.getMessage())
                    .append(";");
        }
        String errorMessage = errorMessageBuilder.toString();
        if (StrUtil.isNotBlank(errorMessage)) {
            return ResponseStatus.fail(errorMessage);
        }
        return ResponseStatus.success();
    }

    @Override
    public ResponseStatus baseApproveNotification(String callbackUrl, ZeroTrustApproveCallbackReqV3VO zeroTrustApproveCallbackReqV3VO) {

        String resultMessage;
        ResponseStatus responseStatus = ResponseStatus.success();
        try {
            logger.info("打印状态变更通知请求:{} 内容：{}", callbackUrl, JsonUtils.toJSONString(zeroTrustApproveCallbackReqV3VO));

            RequestEntity<ZeroTrustApproveCallbackReqV3VO> requestEntity = new RequestEntity<>(zeroTrustApproveCallbackReqV3VO,
                    HttpMethod.POST, URI.create(callbackUrl));
            ResponseEntity<String> responseEntity = restTemplate.exchange(requestEntity, String.class);

            logger.info("打印状态变更通知返回值:" + responseEntity);

            if (responseEntity.getStatusCode().is2xxSuccessful()) {
                String responseEntityBody = responseEntity.getBody();
                ZeroTrustMessageRespVO zeroTrustMessageRespVO = dragonObjectMapper.readValue(responseEntityBody, ZeroTrustMessageRespVO.class);
                if (zeroTrustMessageRespVO == null) {
                    logger.info("打印状态变更通知返回值:" + responseEntity);
                    throw new RemoteAccessException(responseEntity.toString());

                } else if (zeroTrustMessageRespVO.isRespFail()) {
                    logger.info("打印状态变更通知返回值:" + responseEntity);
                    throw new RemoteAccessException(zeroTrustMessageRespVO.getMessage());
                }
            }
        } catch (Exception e) {
            logger.error("状态变更通知接口调用失败", e);
            resultMessage = "状态变更通知接口调用失败：" + e.getMessage();
            responseStatus = ResponseStatus.fail(resultMessage);
            throw new RemoteAccessException(responseStatus.getMessage());
        }
        return responseStatus;
    }

    /**
     * 请求同步到 BU
     *
     * @param buApproveSyncReqVO 同步请求
     * @return 状态
     */
    @Override
    public ResponseStatus retryRequestSyncToBu(BuApproveSyncReqVO buApproveSyncReqVO) {
        if (!approveBuProperties.isEnable()) {
            return ResponseStatus.success();
        }
        String syncTaskUrl = approveBuProperties.getSyncTaskUrl();

        ResponseStatus responseStatus;
        try {
            responseStatus = RESPONSE_STATUS_RETRYER.call(() -> syncToBuBaseReq(syncTaskUrl, buApproveSyncReqVO, "同步任务信息到BU"));
        } catch (ExecutionException | RetryException e) {
            logger.error("retry error", e);
            responseStatus = ResponseStatus.fail(e.getMessage());
        }

        return responseStatus;
    }

    /**
     * 流程详情
     *
     * @param url                路径
     * @param buApproveSyncReqVO 请求参数
     * @return 返回内容
     */
    protected ResponseStatus syncToBuBaseReq(String url, BuApproveSyncReqVO buApproveSyncReqVO, String requestName) {
        // 打印请求参数
        logger.info(requestName + " 请求路径：{} 请求参数:{}", url, JsonUtils.toJSONString(buApproveSyncReqVO));
        String errorTip = requestName + "失败";
        ResponseStatus responseStatus = ValidUtil.validReqVo(buApproveSyncReqVO);
        if (ResponseUtil.isFail(responseStatus)) {
            logger.error("request vo valid error:{}", responseStatus.getMessage());
            return ResponseUtil.dtoFail(responseStatus.getMessage());
        }

        HttpEntity<BuApproveSyncReqVO> entity = new HttpEntity<>(buApproveSyncReqVO);
        ResponseEntity<String> exchange = restTemplate.exchange(url, HttpMethod.POST, entity, String.class);

        if (!exchange.getStatusCode().is2xxSuccessful()) {
            logger.error("{} error. resp: {}", requestName, JsonUtils.toJSONString(exchange));
            return ResponseUtil.dtoFail(errorTip);
        }
        String body = exchange.getBody();
        if (StringUtils.isBlank(body)) {
            logger.error("{} error. resp: {}", requestName, JsonUtils.toJSONString(exchange));
            return ResponseUtil.dtoFail(errorTip);
        }

        logger.info("{} baseReq resp :{}", requestName, JsonUtils.toJSONString(exchange));
        TypeReference<ZeroTrustResultDataVO<Object>> typeReference = new TypeReference<ZeroTrustResultDataVO<Object>>() {
        };

        ZeroTrustResultDataVO<Object> respDto;
        try {
            JsonNode jsonNode = dragonObjectMapper.readTree(body);
            String statusCode = jsonNode.get("status_code").asText();
            String message = jsonNode.get("message").asText();
            if (BuErrorCodeEnum.SUCCESS.getValue().equals(statusCode)) {
                respDto = dragonObjectMapper.readValue(body, typeReference);
            } else {
                return ResponseUtil.dtoFail(message);
            }
        } catch (JsonProcessingException e) {
            logger.error("parse error.", e);
            return ResponseUtil.dtoFail("返回值解析失败");
        }

        return ResponseUtil.newInstance(respDto.getData());
    }
    
    /**
     * V2 版本
     */
	@Override
	public ResponseStatus retryRequestSyncToBuV2(
			com.dragonsoft.dcuc.approve.model.req.bu.v2.BuApproveSyncReqVO buApproveSyncReqVO) {
		if (!approveBuProperties.isEnable()) {
            return ResponseStatus.success();
        }
        String syncTaskUrl = approveBuProperties.getSyncTaskUrl();

        ResponseStatus responseStatus;
        try {
            responseStatus = RESPONSE_STATUS_RETRYER.call(() -> syncToBuBaseReqV2(syncTaskUrl, buApproveSyncReqVO, "同步任务信息到BUV2"));
        } catch (ExecutionException | RetryException e) {
            logger.error("retry error", e);
            responseStatus = ResponseStatus.fail(e.getMessage());
        }

        return responseStatus;
	}

	
	/**
	 * V2 版本
	 * @param url
	 * @param buApproveSyncReqVO
	 * @param requestName
	 * @return
	 */
	protected ResponseStatus syncToBuBaseReqV2(String url, com.dragonsoft.dcuc.approve.model.req.bu.v2.BuApproveSyncReqVO buApproveSyncReqVO, String requestName) {
        // 打印请求参数
        logger.info(requestName + " 请求路径：{} 请求参数:{}", url, JsonUtils.toJSONString(buApproveSyncReqVO));
        String errorTip = requestName + "失败";
        ResponseStatus responseStatus = ValidUtil.validReqVo(buApproveSyncReqVO);
        if (ResponseUtil.isFail(responseStatus)) {
            logger.error("request vo valid error:{}", responseStatus.getMessage());
            return ResponseUtil.dtoFail(responseStatus.getMessage());
        }

        HttpEntity<com.dragonsoft.dcuc.approve.model.req.bu.v2.BuApproveSyncReqVO> entity = new HttpEntity<>(buApproveSyncReqVO);
        ResponseEntity<String> exchange = restTemplate.exchange(url, HttpMethod.POST, entity, String.class);

        if (!exchange.getStatusCode().is2xxSuccessful()) {
            logger.error("{} error. resp: {}", requestName, JsonUtils.toJSONString(exchange));
            return ResponseUtil.dtoFail(errorTip);
        }
        String body = exchange.getBody();
        if (StringUtils.isBlank(body)) {
            logger.error("{} error. resp: {}", requestName, JsonUtils.toJSONString(exchange));
            return ResponseUtil.dtoFail(errorTip);
        }

        logger.info("{} baseReq resp :{}", requestName, JsonUtils.toJSONString(exchange));
        TypeReference<ZeroTrustResultDataVO<Object>> typeReference = new TypeReference<ZeroTrustResultDataVO<Object>>() {
        };

        ZeroTrustResultDataVO<Object> respDto;
        try {
            JsonNode jsonNode = dragonObjectMapper.readTree(body);
            String statusCode = jsonNode.get("status_code").asText();
            String message = jsonNode.get("message").asText();
            if (BuErrorCodeEnum.SUCCESS.getValue().equals(statusCode)) {
                respDto = dragonObjectMapper.readValue(body, typeReference);
            } else {
                return ResponseUtil.dtoFail(message);
            }
        } catch (JsonProcessingException e) {
            logger.error("parse error.", e);
            return ResponseUtil.dtoFail("返回值解析失败");
        }

        return ResponseUtil.newInstance(respDto.getData());
    }
    
}
