package com.dragonsoft.dcuc.approve.business.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.dragonsoft.dcuc.approve.business.ICacheBusiness;
import com.dragonsoft.dcuc.approve.constants.ApproveRedisConstants;
import com.dragonsoft.dcuc.approve.model.dto.zerotrust.AppTokenInfoDTO;
import com.dragonsoft.dcuc.approve.model.dto.zerotrust.UserTokenInfoDTO;
import com.dragonsoft.dcuc.approve.enumresources.ZeroTrustTokenActionEnum;
import com.dragonsoft.duceap.commons.util.date.DateConst;
import com.dragonsoft.duceap.commons.util.date.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.util.Date;
import java.util.concurrent.TimeUnit;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/7/13
 */
@Slf4j
@Component
public class RedisCacheBusinessImpl implements ICacheBusiness {

    private StringRedisTemplate stringRedisTemplate;

    @Autowired
    public void setStringRedisTemplate(StringRedisTemplate stringRedisTemplate) {
        this.stringRedisTemplate = stringRedisTemplate;
    }

    @Override
    public void saveOrgSyncTime(Date date) {
        String dateFormat = DateUtils.getDateFormat(date, DateConst.DB_STORE_TIME);
        stringRedisTemplate.opsForValue().set(ApproveRedisConstants.SYNC_ORG_TIME, dateFormat);
    }

    @Override
    public void saveUserSyncTime(Date date) {
        String dateFormat = DateUtils.getDateFormat(date, DateConst.DB_STORE_TIME);
        stringRedisTemplate.opsForValue().set(ApproveRedisConstants.SYNC_USER_TIME, dateFormat);
    }

    @Override
    public Date getLastOrgSyncTime() {
        String dateString = stringRedisTemplate.opsForValue().get(ApproveRedisConstants.SYNC_ORG_TIME);
        if (StrUtil.isBlank(dateString)) {
            return null;
        }
        try {
            return DateUtils.getDate(dateString, DateConst.DB_STORE_TIME);
        } catch (ParseException e) {
            log.error("parse date ", e);
            return null;
        }
    }

    @Override
    public Date getLastUserSyncTime() {
        String dateString = stringRedisTemplate.opsForValue().get(ApproveRedisConstants.SYNC_USER_TIME);
        if (StrUtil.isBlank(dateString)) {
            return null;
        }
        try {
            return DateUtils.getDate(dateString, DateConst.DB_STORE_TIME);
        } catch (ParseException e) {
            log.error("parse date ", e);
            return null;
        }
    }


    @Override
    public void appTokenActionHandle(AppTokenInfoDTO tokenInfo, String action) {
        if (null == tokenInfo) {
            return;
        }
        if (ZeroTrustTokenActionEnum.OFFLINE.getValue().equals(action)) {
            deleteAppToken(tokenInfo);
        } else if (ZeroTrustTokenActionEnum.ONLINE.getValue().equals(action)) {
            cacheAppToken(tokenInfo);
        } else if (ZeroTrustTokenActionEnum.RENEW.getValue().equals(action)) {
            renewAppToken(tokenInfo);
        }
    }

    private void cacheAppToken(AppTokenInfoDTO appToken) {
        log.info("缓存的应用令牌 令牌id:{}, appId:{}", appToken.getAppTokenId(), appToken.getAppId());

        long tokenExpireTime = appToken.getExpireAt().getTime();
        long redisExpire = (tokenExpireTime - System.currentTimeMillis()) / 1000;
        stringRedisTemplate.opsForValue().set(appToken.getAppTokenId(), JSON.toJSONString(appToken), redisExpire, TimeUnit.SECONDS);
    }

    private void renewAppToken(AppTokenInfoDTO tokenInfo) {
        log.info("续期应用令牌");
        cacheAppToken(tokenInfo);
    }

    private void deleteAppToken(AppTokenInfoDTO tokenInfo) {
        log.info("删除缓存的应用令牌 令牌id:{}, appId:{}", tokenInfo.getAppTokenId(), tokenInfo.getAppId());
        stringRedisTemplate.delete(tokenInfo.getAppTokenId());
    }

    @Override
    public void userTokenActionHandle(UserTokenInfoDTO tokenInfo, String action) {
        if (null == tokenInfo) {
            return;
        }
        if (ZeroTrustTokenActionEnum.OFFLINE.getValue().equals(action)) {
            deleteUserToken(tokenInfo);
        } else if (ZeroTrustTokenActionEnum.ONLINE.getValue().equals(action)) {
            cacheUserToken(tokenInfo);
        } else if (ZeroTrustTokenActionEnum.RENEW.getValue().equals(action)) {
            renewUserToken(tokenInfo);
        }
    }

    private void renewUserToken(UserTokenInfoDTO tokenInfo) {
        log.info("续期用户令牌");
        cacheUserToken(tokenInfo);
    }

    private void cacheUserToken(UserTokenInfoDTO tokenInfo) {
        log.info("缓存用户令牌 令牌id:{}, pid:{}", tokenInfo.getUserTokenId(), tokenInfo.getPid());
        // 缓存人员令牌
        long tokenExpireTime = tokenInfo.getExpireAt().getTime();
        long redisExpire = (tokenExpireTime - System.currentTimeMillis()) / 1000;
        stringRedisTemplate.opsForValue().set(tokenInfo.getUserTokenId(), JSON.toJSONString(tokenInfo), redisExpire, TimeUnit.SECONDS);

    }

    private void deleteUserToken(UserTokenInfoDTO tokenInfo) {
        log.info("删除缓存的用户令牌 令牌id:{}, pid:{}", tokenInfo.getUserTokenId(), tokenInfo.getPid());
        // 删除令牌
        stringRedisTemplate.delete(tokenInfo.getUserTokenId());
    }

}
