package com.dragonsoft.dcuc.approve.utils;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.dragoninfo.dcuc.common.Constants;
import com.dragoninfo.dcuc.common.entity.Proofs;
import com.dragoninfo.dcuc.common.enums.UserExtInfoEnum;
import com.dragoninfo.dcuc.common.utils.SecurityUserUtil;
import com.dragonsoft.duceap.base.entity.security.BaseSecurityUser;
import com.dragonsoft.duceap.base.entity.security.SecurityUser;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.util.Base64Utils;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;
import java.util.Optional;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2021/8/25
 */
public class ProofsUtil {
    private ProofsUtil() {
    }

    public static String getIdcard() {
        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        if (currentUser != null) {
            return currentUser.getIdcard();
        }
        return "";
    }


    public static Proofs generalProofs(HttpServletRequest request, String defaultAppCode) {
        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        String idcard;
        String appCode;
        if (currentUser != null) {
            idcard = currentUser.getIdcard();
            appCode = defaultAppCode;
        } else {
            idcard = getIdcard(request);
            appCode = getAppCode(request);
        }

        return Proofs.builder()
                .appCode(appCode)
                .idcard(idcard)
                .userToken(getUserToken(request))
                .appToken(getAppToken(request))
                .build();
    }

    public static String getUserToken(HttpServletRequest request) {
        // 使用Header获取的key会全部转换为小写字母
        String userToken = request.getHeader(Constants.USER_TOKEN);

        if (StrUtil.isBlank(userToken)) {
            userToken = request.getHeader(Constants.BUS_SRE_TOKEN);
        }

        if (StrUtil.isBlank(userToken)) {
            userToken = request.getHeader(Constants.RZZX_USERTOKEN);
        }

        return userToken;
    }

    public static String getAppToken(HttpServletRequest request) {
        String appToken = request.getHeader(Constants.APP_TOKEN);
        if (StrUtil.isBlank(appToken)) {
            appToken = request.getHeader(Constants.BUS_SRA_TOKEN);
        }

        if (StrUtil.isBlank(appToken)) {
            appToken = request.getHeader(Constants.RZZX_APPTOKEN);
        }
        return appToken;
    }

    /**
     * 获取身份证号
     *
     * @param request 请求头
     * @return 身份证号
     */
    public static String getIdcard(HttpServletRequest request) {
        // 使用Header获取的key会全部转换为小写字母
        String userToken = getUserToken(request);
        String idcard = "";
        if (StrUtil.isNotBlank(userToken)) {
            Optional<JSONObject> jsonObject = parseJwt(userToken);
            if (jsonObject.isPresent()) {
                idcard = jsonObject.get().getString("idCard");
            }
        }

        if (StrUtil.isBlank(idcard)) {
            idcard = request.getHeader(Constants.IDCARD);
        }

        if (StrUtil.isBlank(idcard)) {
            idcard = request.getHeader(Constants.BUS_SRE_IDCARD);
        }

        return idcard;
    }

    /**
     * 获取应用代码
     *
     * @param request 请求头
     * @return 应用代码
     */
    public static String getAppCode(HttpServletRequest request) {
        String appToken = getAppToken(request);

        String appCode = "";
        if (StrUtil.isNotBlank(appToken)) {
            Optional<JSONObject> jsonObject = parseJwt(appToken);
            if (jsonObject.isPresent()) {
                appCode = jsonObject.get().getString("appCode");
            }
        }

        if (StrUtil.isBlank(appCode)) {
            appCode = request.getHeader(Constants.APP_CODE);
        }

        if (StrUtil.isBlank(appCode)) {
            appCode = request.getHeader(Constants.BUS_SRA_ID);
        }

        return appCode;
    }

    /**
     * 解析JWT内容
     *
     * @param token jwtToken
     * @return 负载内容
     */
    private static Optional<JSONObject> parseJwt(String token) {
        if (StrUtil.isBlank(token)) {
            return Optional.empty();
        }
        int jwtLength = 3;
        String[] split = token.split("\\.");
        if (split.length == jwtLength) {
            String playLoad = split[1];
            byte[] bytes = Base64Utils.decodeFromUrlSafeString(playLoad);
            JSONObject jsonObject = JsonUtils.parseObject(new String(bytes));
            return Optional.of(jsonObject);
        }
        return Optional.empty();
    }

    /**
     * 生成请求头
     *
     * @return 请求头信息
     */
    public static HttpHeaders buildHttpHeaders() {
        SecurityUser currentUser = UserContextUtils.getCurrentUser();
        return buildHttpHeaders(currentUser);
    }

    /**
     * 生成请求头信息
     *
     * @param currentUser 当前用户
     * @return 请求头信息
     */
    public static HttpHeaders buildHttpHeaders(SecurityUser currentUser) {
        Assert.notNull(currentUser);
        Map<String, Object> extendtions = currentUser.getExtendtions();

        HttpHeaders headers = new HttpHeaders();
        if (MapUtil.isEmpty(extendtions)) {
            return headers;
        }

        Object userTokenObject = extendtions.get(UserExtInfoEnum.USER_TOKEN_ID.getValue());
        String userToken = "";
        if (ObjectUtil.isNotNull(userTokenObject)) {
            userToken = userTokenObject.toString();
        }
        Object appTokenObject = extendtions.get(UserExtInfoEnum.APP_TOKEN_ID.getValue());
        String appToken = "";
        if (ObjectUtil.isNotNull(appTokenObject)) {
            appToken = appTokenObject.toString();
        }

        String idcard = currentUser.getIdcard();
        String appCode = SecurityUserUtil.getSecurityAppCode();

        if (StrUtil.isNotBlank(appToken)) {
            //应用token
            headers.add("appToken", appToken);
        }
        if (StrUtil.isNotBlank(userToken)) {
            //用户token
            headers.add("userToken", userToken);
        }

        if (StrUtil.isNotBlank(idcard)) {
            headers.add("idcard", idcard);
        }

        if (StrUtil.isNotBlank(appCode)) {
            headers.add("appCode", appCode);
        }

        return headers;
    }
}
